#!/bin/bash
# smpproutercli.sh - Command-line tool for interacting with SMPP Router API
# Author: Melrose Labs
# Description: A shell script to manage ESMEs, target sets, and configuration via HTTP API.
# Dependencies: bash, curl, jq, tr, grep, cut, awk, sed, printf, xargs

set -e

# Check for required commands
for cmd in jq curl; do
  if ! command -v "$cmd" >/dev/null 2>&1; then
    echo "Error: Required command '$cmd' not found. Please install it and retry."
    exit 1
  fi
done

# Load configuration defaults from ~/.smpprouterclirc if it exists
CONFIG_FILE="$HOME/.smpprouterclirc"
if [[ -f "$CONFIG_FILE" ]]; then
  # shellcheck source=/dev/null
  source "$CONFIG_FILE"
fi

if [[ $# -eq 0 ]]; then
  echo "smpproutercli - Command-line tool for interacting with SMPP Router API"
  echo "Copyright (c) Melrose Labs. All rights reserved."
  echo "For usage instructions:"
  echo "  smpproutercli --help"
  exit 0
fi

print_status() {
  local code="${1}"
  local status_line="${2}"
  local success_prefix="${SUCCESS_PREFIX:-SUCCESS:}"
  local error_prefix="${ERROR_PREFIX:-ERROR:}"
  if [[ "${status_line}" == *"400 "* ]]; then
    local error_msg=$(echo "${status_line}" | cut -d' ' -f3-)
    echo -e "${error_prefix} ${error_msg}"
  elif [[ "${code}" =~ ^4[0-9][0-9]$ ]]; then
    local error_msg=$(echo "${status_line}" | cut -d' ' -f3-)
    echo -e "${error_prefix} ${error_msg}"
  elif [[ "${code}" =~ ^2[0-9][0-9]$ ]]; then
    local success_msg=$(echo "${status_line}" | cut -d' ' -f3-)
    echo -e "${success_prefix} ${success_msg}"
  elif [[ "${code}" != 2* ]]; then
    echo "HTTP Error ${code}: ${status_line}"
  fi
}

# Helper: parse response into STATUS_LINE, CODE, BODY
parse_response_components() {
  local response="${1}"
  STATUS_LINE=$(echo "${response}" | head -n 1)
  CODE=$(echo "${STATUS_LINE}" | awk '{print $2}')
  BODY=$(echo "${response}" | sed '1,/^\r*$/d')
}

# Perform curl, return response, detect errors
perform_curl() {
  local method="${1}"
  local url="${2}"
  local data="${3}"
  local response
  local curl_status

  if [[ -n "${data}" ]]; then
    response=$(curl -s -i -H "${AUTH_HEADER}" -H "${CONTENT_HEADER}" -X "${method}" "${url}" -d "${data}")
    curl_status=$?
  else
    response=$(curl -s -i -H "${AUTH_HEADER}" -H "${CONTENT_HEADER}" -X "${method}" "${url}")
    curl_status=$?
  fi

  if [[ ${curl_status} -ne 0 ]]; then
    echo -e "${ERROR_PREFIX} Failed to connect to ${url} (curl exit code: ${curl_status})"
    exit 1
  fi

  echo "${response}"
}

# Unified response handler
handle_response() {
  local response="${1}"
  parse_response_components "${response}"
  if [[ "${SHOW_RAW_JSON}" == "true" ]]; then
    echo "${BODY}"
  fi
  print_status "${CODE}" "${STATUS_LINE}"
}


# Colour codes
GREEN='\033[0;32m'
RED='\033[0;31m'
NC='\033[0m' # No Color

# Prefixes for success and error messages
SUCCESS_PREFIX="${GREEN}SUCCESS:${NC}"
ERROR_PREFIX="${RED}ERROR:${NC}"

# Set to "true" to print raw JSON responses
SHOW_RAW_JSON=false

kv_to_json() {
  local first=1
  printf '{'
  for kv in "$@"; do
    local key=${kv%%=*}
    local value=${kv#*=}
    # Special fields always as string
    case "${key}" in
      inboundmo|allowedips)
        : # Use value as-is (verbatim)
        ;;
      targetset|password|systemid|id|name|systemtype|description)
        value="\"${value}\""
        ;;
      *)
        if [[ "${value}" =~ ^[0-9]+$ && ! "${value}" =~ ^0[0-9]+$ || "${value}" == "true" || "${value}" == "false" ]]; then
          :
        else
          value="\"${value}\""
        fi
        ;;
    esac
    if [[ ${first} -eq 1 ]]; then
      first=0
    else
      printf ','
    fi
    printf '"%s":%s' "${key}" "${value}"
  done
  printf '}\n'
}

if [[ -n "${CLIENT_ID}" && -n "${CLIENT_SECRET}" ]]; then
  TOKEN_RESPONSE=$(curl -s -H "Content-Type: application/x-www-form-urlencoded" \
    -d "grant_type=client_credentials&scope=api&client_id=${CLIENT_ID}&client_secret=${CLIENT_SECRET}" \
    http://${SMPPROUTERHOST:-localhost:80}/oauth2/v1/token)
  curl_status=$?
  if [[ ${curl_status} -ne 0 ]]; then
    echo -e "${ERROR_PREFIX} Failed to connect to http://${SMPPROUTERHOST:-localhost:80}/oauth2/v1/token (curl exit code: ${curl_status})"
    exit 1
  fi
  ACCESS_TOKEN=$(echo "${TOKEN_RESPONSE}" | grep -o '"access_token":"[^"]*' | cut -d':' -f2 | tr -d '"')
  if [[ -z "${ACCESS_TOKEN}" ]]; then
    echo -e "${ERROR_PREFIX} Failed to retrieve access token"
    exit 1
  fi
  AUTH_HEADER="Authorization: Bearer ${ACCESS_TOKEN}"
else
  AUTH_HEADER="Authorization: Bearer abc"
fi
CONTENT_HEADER="Content-Type: application/json"
HOST=${SLBMHOST:-localhost:80}

if [[ $# -gt 0 ]]; then
  cmd="${1}"
  shift
  case "${cmd}" in
    --help)
      cat <<EOF
Usage: $0 <command> [arguments]

Commands:

  -- CONFIGURE --

  add-esme <key=value>...           Add a new ESME with specified properties.
  mod-esme <systemid> <key=value>...      Modify an existing ESME by ID.
  list-esme                         List all ESMEs.
  get-esme <systemid>               Get details of an ESME by ID.
  del-esme <systemid>               Delete an ESME by ID.

  add-targetset <key=value>...      Add a new target set.
  mod-targetset <ts_id> <key=value>... Modify an existing target set by ID.
  list-targetsets                   List all target sets.
  get-targetset <ts_id>             Get details of a target set by ID.
  del-targetset <ts_id>             Delete a target set by ID.
  
  add-target <ts_id> <key=value>... Add a target to a target set.
  mod-target <ts_id> <t_name> <key=value>... Modify a target in a target set.
  list-targets <ts_id>              List targets in a target set.
  get-target <ts_id> <t_name>       Get details of a target in a target set.
  del-target <ts_id> <t_name>       Delete a target from a target set.
  
  -- CONTROL --
  
  control log.audit=<value>         Enable (true) or disable (false) audit log (does not change configured value)
  
  -- STATUS --
  
  status                            Display target status

Environment variables and config file (~/.smpprouterclirc):
  CLIENT_ID, CLIENT_SECRET          Credentials for OAuth2 authentication.
  SMPPROUTERHOST                    Host for SMPP Router OAuth2 token endpoint and API (default: localhost:80).

Examples:
  $0 add-esme systemid=esme1 password=pass123 targetset=ts001 tpslimit=100
  $0 mod-esme esme1 password=newpass
  $0 list-esme
  $0 get-esme esme1
  $0 del-esme esme1
  
  $0 add-targetset id=ts001 name=TS001 submits_enabled=true
  $0 mod-targetset ts001 name=TS001 submits_enabled=false
  $0 list-targetsets
  $0 get-targetset ts001
  $0 del-targetset ts001
  
  $0 add-target ts001 host=smscsim.melroselabs.com port=2775 systemid=123456 password=123456 name=smsc7
  $0 mod-target ts001 smsc3 admindown=true
  $0 list-target ts001 smsc3
  $0 get-target ts001 smsc3
  $0 del-target ts001 smsc3

Use --help to display this message.

EOF
      exit 0
      ;;
    add-esme)
      payload=$(kv_to_json "$@") && echo "$payload" | jq . >/dev/null || { echo "ERROR: Invalid JSON"; exit 1; }
      RESPONSE=$(perform_curl POST "http://${HOST}/esme" "${payload}")
      handle_response "${RESPONSE}"
      exit 0
      ;;
    mod-esme)
      id="${1}"
      shift
      payload=$(kv_to_json "$@") && echo "$payload" | jq . >/dev/null || { echo "ERROR: Invalid JSON"; exit 1; }
      RESPONSE=$(perform_curl PUT "http://${HOST}/esme/${id}" "${payload}")
      handle_response "${RESPONSE}"
      exit 0
      ;;
    mod-targetset)
      id="${1}"
      shift
      payload=$(kv_to_json "$@") && echo "$payload" | jq . >/dev/null || { echo "ERROR: Invalid JSON"; exit 1; }
      RESPONSE=$(perform_curl PUT "http://${HOST}/targetsets/${id}" "${payload}")
      handle_response "${RESPONSE}"
      exit 0
      ;;
    add-target)
      tsid="${1}"
      shift
      payload=$(kv_to_json "$@") && echo "$payload" | jq . >/dev/null || { echo "ERROR: Invalid JSON"; exit 1; }
      RESPONSE=$(perform_curl POST "http://${HOST}/targetsets/${tsid}/targets" "${payload}")
      handle_response "${RESPONSE}"
      exit 0
      ;;
    mod-target)
      tsid="${1}"
      target="${2}"
      shift 2
      payload=$(kv_to_json "$@") && echo "$payload" | jq . >/dev/null || { echo "ERROR: Invalid JSON"; exit 1; }
      RESPONSE=$(perform_curl PUT "http://${HOST}/targetsets/${tsid}/targets/${target}" "${payload}")
      handle_response "${RESPONSE}"
      exit 0
      ;;
    list-esme)
      RESPONSE=$(perform_curl GET "http://${HOST}/esme")
      handle_response "${RESPONSE}"
      parse_response_components "${RESPONSE}"
      if [[ "${CODE}" =~ ^2 ]]; then
        echo "ESMEs:"
        echo "-------------------------"
        echo "${BODY}" | jq -r '.[] | "- \(.)"'
      fi
      exit 0
      ;;
    del-esme)
      id="${1}"
      RESPONSE=$(perform_curl DELETE "http://${HOST}/esme/${id}")
      handle_response "${RESPONSE}"
      exit 0
      ;;
    get-esme)
      id="${1}"
      RESPONSE=$(perform_curl GET "http://${HOST}/esme/${id}")
      handle_response "${RESPONSE}"
      parse_response_components "${RESPONSE}"
      if [[ "${CODE}" =~ ^2 ]]; then
        echo "ESME ${id}:"
        echo "-------------------------"
        # Try to pretty-print JSON if valid, else print raw body
        if echo "${BODY}" | jq empty 2>/dev/null; then
          echo "${BODY}" | jq -r '
            to_entries[] |
            if (.value | type == "array") then
              "\(.key):",
              .value[] |
                "  -------------------------",
                if type == "object" then
                  to_entries[] | "  \(.key): \(.value)"
                else
                  "  - \(.)"
                end
            elif (.value | type == "object") then
              "\(.key):",
              .value | to_entries[] | "  \(.key): \(.value)"
            else
              "\(.key): \(.value)"
            end
          '
        else
          echo "${BODY}"
        fi
      fi
      exit 0
      ;;
    add-targetset)
      payload=$(kv_to_json "$@") && echo "$payload" | jq . >/dev/null || { echo "ERROR: Invalid JSON"; exit 1; }
      RESPONSE=$(perform_curl POST "http://${HOST}/targetsets" "${payload}")
      handle_response "${RESPONSE}"
      exit 0
      ;;
    list-targetsets)
      RESPONSE=$(perform_curl GET "http://${HOST}/targetsets")
      handle_response "${RESPONSE}"
      parse_response_components "${RESPONSE}"
      if [[ "${CODE}" =~ ^2 ]]; then
        echo "Target Sets:"
        echo "${BODY}" | jq -r '.[] | "- \(.)"'
      fi
      exit 0
      ;;
    get-targetset)
      id="${1}"
      RESPONSE=$(perform_curl GET "http://${HOST}/targetsets/${id}")
      handle_response "${RESPONSE}"
      parse_response_components "${RESPONSE}"
      if [[ "${CODE}" =~ ^2 ]]; then
        echo "Target Set ${id}:"
        echo "-------------------------"
        if [[ "${SHOW_RAW_JSON}" == "true" ]]; then
          echo "${BODY}"
        fi
        if echo "${BODY}" | jq empty 2>/dev/null; then
          echo "${BODY}" | jq -r '
            to_entries[] |
            if (.value | type == "array") then
              "\(.key):" ,
              .value[] |
                "  -------------------------",
                if type == "object" then
                  to_entries[] | "  \(.key): \(.value)"
                else
                  "  - \(.)"
                end
            elif (.value | type == "object") then
              "\(.key):",
              .value | to_entries[] | "  \(.key): \(.value)"
            else
              "\(.key): \(.value)"
            end
          '
        else
          echo "${BODY}"
        fi
      fi
      exit 0
      ;;
    get-target)
      tsid="${1}"
      target="${2}"
      RESPONSE=$(perform_curl GET "http://${HOST}/targetsets/${tsid}/targets/${target}")
      handle_response "${RESPONSE}"
      parse_response_components "${RESPONSE}"
      if [[ "${CODE}" =~ ^2 ]]; then
        echo "Target '${target}' in Target Set ${tsid}:"
        echo "-------------------------"
        if echo "${BODY}" | jq empty 2>/dev/null; then
          echo "${BODY}" | jq -r '
            to_entries[] |
            if (.value | type == "array") then
              "\(.key):" ,
              .value[] |
                "  -------------------------",
                if type == "object" then
                  to_entries[] | "  \(.key): \(.value)"
                else
                  "  - \(.)"
                end
            elif (.value | type == "object") then
              "\(.key):",
              .value | to_entries[] | "  \(.key): \(.value)"
            else
              "\(.key): \(.value)"
            end
          '
        else
          echo "${BODY}"
        fi
      fi
      exit 0
      ;;
    del-targetset)
      id="${1}"
      RESPONSE=$(perform_curl DELETE "http://${HOST}/targetsets/${id}")
      handle_response "${RESPONSE}"
      exit 0
      ;;
    list-targets)
      tsid="${1}"
      RESPONSE=$(perform_curl GET "http://${HOST}/targetsets/${tsid}/targets")
      handle_response "${RESPONSE}"
      parse_response_components "${RESPONSE}"
      if [[ "${CODE}" =~ ^2 ]]; then
        echo "Targets in Target Set ${tsid}:"
        echo "-------------------------"
        echo "${BODY}" | jq -r '.[] | "- \(.)"'
      fi
      exit 0
      ;;
    del-target)
      tsid="${1}"
      target="${2}"
      RESPONSE=$(perform_curl DELETE "http://${HOST}/targetsets/${tsid}/targets/${target}")
      parse_response_components "${RESPONSE}"
      echo "${BODY}"
      print_status "${CODE}" "${STATUS_LINE}"
      exit 0
      ;;

    control)
      payload=$(kv_to_json "$@") && echo "$payload" | jq . >/dev/null || { echo "ERROR: Invalid JSON"; exit 1; }
      RESPONSE=$(perform_curl POST "http://${HOST}/control" "${payload}")
      handle_response "${RESPONSE}"
      exit 0
      ;;

    status)
      RESPONSE=$(perform_curl GET "http://${HOST}/status")
      handle_response "${RESPONSE}"
      parse_response_components "${RESPONSE}"
      if [[ "${CODE}" =~ ^2 ]]; then
        echo "Status:"
        echo "-------------------------"
        #echo "${BODY}" | jq -r '.[] | "- \(.)"'
        echo "${BODY}" | jq -r '.statusReport |
          def colour(text; code): "\u001b[" + code + "m" + text + "\u001b[0m";
          
          def bind_type_label(type):
            if type == 0 then "RX"
            elif type == 1 then "TX"
            elif type == 2 then "TRX"
            else "UNKNOWN"
            end;
            
          def bind_state_label(state):
            if state == 0 then "UNBOUND"
            elif state == 1 then "BINDING"
            elif state == 2 then "BOUND_TRX"
            elif state == 3 then "BOUND_TX"
            elif state == 4 then "BOUND_RX"
            elif state == 5 then "UNBINDING"
            else "UNKNOWN"
            end;

          sort_by(.target_set, .smsc_name, .smsc_id)
          | group_by(.target_set)
          | .[] as $ts_group |
            (
              "Target set: \($ts_group[0].target_set)\n" +
              (
                $ts_group
            | group_by(.smsc_name)
            | .[] as $group |
            (
              " " +
              colour("Target: \($group[0].smsc_name) [Base ID: \($group[0].smsc_id / 256 | floor)]"; "30") + "\n" +
              colour("  Host: \($group[0].host):\($group[0].port)\t  System ID: \($group[0].system_id)"; "90") + "\n" +
              (
                $group
                | map(
                    if .admin_down then
                      "  \u001b[33m\u2588 \u001b[0m[ID: \(.smsc_id % 256)] Connected: "+
                      colour("\(.connected)"; "31")+
                      ", Bind Type: \(bind_type_label(.bind_type)), Bind State:"+
                      colour(" \(bind_state_label(.bind_state))"; "31")+
                      ", Admin Down: \u001b[43m\(.admin_down)\u001b[0m"
                      
                    elif (.bind_state == 0 or .bind_state == 5) then

                      "  \u001b[31m\u2588 \u001b[0m[ID: \(.smsc_id % 256)] Connected: "+
                      colour("\(.connected)"; "31")+
                      ", Bind Type: \(bind_type_label(.bind_type)), Bind State:"+
                      colour(" \(bind_state_label(.bind_state))"; "31")+
                      ", Admin Down: \(.admin_down) "
                    
                    else
                      "  \u001b[32m\u2588 \u001b[0m[ID: \(.smsc_id % 256)] Connected: "+
                      colour("\(.connected)"; "32")+
                      ", Bind Type: \(bind_type_label(.bind_type)), Bind State:"+
                      colour(" \(bind_state_label(.bind_state))"; "32")+
                      ", Admin Down: \(.admin_down)"
                    end
                  )
                | join("\n")
              ) + "\n"
            )
          )
          )
        
        '
        echo "${BODY}" | jq -r '.summary |
         "----\n" +
         "Admin Down: \(.admindown)  Targets: \(.targets)  Target sets: \(.targetsets)  Bind TX: \(.bindTX)  Bind RX: \(.bindRX)  Bind TRX: \(.bindTRX)  Unbound: \(.bindUnbound)" +
         "\n----"
        '
      fi
      exit 0
      ;;

    *)
      echo "Unknown command: ${cmd}"
      exit 1
      ;;
      
  esac
fi
